/***************************************************************************
                          kpgconnection.h  -  description
                             -------------------
    begin                : � led 13 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPGCONNECTION_H
#define KPGCONNECTION_H

// Note that for new code which you wish to be strictly Unicode-clean, you can define the macro QT_NO_ASCII_CAST when compiling your code.
#define QT_NO_ASCII_CAST

// include files for libpgxx 
#include <pqxx/connection>
#include <pqxx/transaction_base> 

#include "kpgfunction.h"
#include "../kpgnoticer.h"

class QTextCodec;

using namespace pqxx;


/**
  * Item that represent connection to PostgreSQL server
  *
  * @author Lumir Vanek
  */
class KPGConnection : public connection
{
public: 
	KPGConnection(const QString);
	virtual ~KPGConnection() throw ();
	
	// Various transactions types
	enum ETransType { eTransNormal = 0, eTransNone, eTransRobust };
	
    void setPassword(const QString & strPassword) { m_strPassword = strPassword; }
    
    const QString & password() const { return m_strPassword; }
  
	// Return encoder name for QTextCodec regards to connection client encoding
	const QString encoderName();
	
	// Return connection encoding
	const QString clientEncoding() { return QString(get_variable("CLIENT_ENCODING")); }
	
	// Return text codec
	QTextCodec *textCodec() const  { return m_pTextCodec; }
	
	// Return given string converted to UNICODE using current codec 
	QString toUnicode(const char * pszString) const;
	
	// Run SQL statement, make conversion regard connection encoding
	pqxx::result runQuery(const QString&);

	// Run SQL statement, make conversion regard connection encoding
	pqxx::result runQuery(const QString &, ETransType);
	
	// Run prepared SQL statement
    pqxx::result runPreparedStatement(const std::string &, const ListFunctionArgumentsWithValues &, ETransType = eTransNormal);
	
    void setTrace();
  
  	// Set prepare declaration for argument, regards of its datatype
    void setPrepareDeclaration(pqxx::prepare::declaration &, const QString &);
      
  	// Set value to prepared statement invocation, regards of its datatype. Return false, if conversion fails
  	bool setInvocationValue(prepare::invocation &, const QString &, const QString &);

	// Return the KPoGre noticer
	KPGNoticer * kpgNoticer() const { return m_pNoticer; }
	
protected:
    
	// Tracing file
	FILE *m_pFileTrace;
    
    // Remember password, that is used for connection
	QString m_strPassword; 
    
    // Codec for encode/decode strings from/to database
    QTextCodec *m_pTextCodec;
	
	// The noticer
	KPGNoticer *m_pNoticer;
};

#endif
